package communicationservices;

import dataservices.*;
import java.util.*;

/**
 * <p>Title: POMESClient </p>
 * <p>Description: Used by sessions to communicate with POMES.
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Printronix</p>
 * @author Christopher M. King (kingc@uci.edu) and Susan Hu (xhu@uci.edu)
 * @version 2.0
 */

import java.io.*;
import java.net.*;

public class POMESClient
{
  // Load some defaults
  private static String pomesIP;
  private static int pomesPort;
  private static String name;
  private static String password;

  private boolean sessionOpen     = false;
  private Socket XMLSocket        = null;
  private DataOutputStream out    = null;
  private BufferedReader in       = null;
  private boolean debug           = false;
  private boolean connected       = false;
  private XMLCommandProcessor xmlCmdProc;
  private POMESComManager pomesComManager;

  public static final String IO_ERROR   = "Error: Couldn't get I/O for connection";
  public static final String HOST_ERROR = "Error: Couldnt find host";

  public POMESClient(POMESComManager pCM, String ip, int port, String newName, String newPassword)
  {
    pomesComManager = pCM;
    pomesIP = ip;
    pomesPort = port;
    name = newName;
    password = newPassword;
    xmlCmdProc = new XMLCommandProcessor();
  }

  public POMESClient(POMESComManager pCM, String ip, int port, String newName, String newPassword, boolean debugFlag)
  {
    pomesComManager = pCM;
    pomesIP = ip;
    pomesPort = port;
    name = newName;
    password = newPassword;
    debug = debugFlag;
    xmlCmdProc = new XMLCommandProcessor();
  }

  public boolean openPOMESSession()
  {
    try
    {
      XMLSocket = new Socket(pomesIP, pomesPort);
      out       = new DataOutputStream(XMLSocket.getOutputStream());
      in        = new BufferedReader(
	          new InputStreamReader(XMLSocket.getInputStream()));

      this.authenticate();
      sessionOpen = true;
      return true;
    }
    catch(UnknownHostException e)
    {
      System.out.println(HOST_ERROR + " at " + pomesIP);
      return false;
    }
    catch(IOException e)
    {
      System.out.println(IO_ERROR + " to " + pomesIP + " on port " + pomesPort);
      return false;
    }
  }

  public void authenticate()
  {
    Vector attrNames = new Vector();
    attrNames.add("name");
    attrNames.add("user");
    attrNames.add("password");

    Vector attrVals = new Vector();
    attrVals.add("Authenticate");
    attrVals.add(name);
    attrVals.add(password);

    String xmlToSend = xmlCmdProc.formCommand(attrNames, attrVals);
    this.sendCommand(xmlToSend);
    this.receiveResponse();
  }

  public boolean closePOMESSession()
  {
    try
    {
      sessionOpen = false;
      out.close();
      in.close();
      XMLSocket.close();
    }
    catch (IOException e)
    {
      System.out.println(e.toString());
      return false;
    }
    return true;
  }

  public boolean POMESSessionOpen()
  {
    return sessionOpen;
  }

  public boolean sendCommand(String data)
  {
    if(out != null)
    {
      try
      {
          out.writeBytes(data);

          if(!connected) // quick fix: Recieves the CONNECTION_RESPONSE if this
          {  this.receiveResponse();// is the first command sent
             connected = true;
          }
      }
      catch (IOException e)
      {
          System.out.println(e.toString());
          return false;
      }
      return true;
    }
    return false;
  }
  public String receiveResponse()
  {
    String line = "";
    String xml = "";
    try
    {
      while((line = in.readLine()) != null)
      {
        xml += line + "\n";
        if(line.indexOf("</POM-API>") != -1)
        {
          pomesComManager.recievedXML(xml);
          return xml;
        }
      }
      in.skip(1);
      xml += "\n";
    }
    catch (IOException e)
    {
      System.out.println(e.toString());
    }
    return xml;
  }
}