package controlservices;

import communicationservices.*;
import dataservices.*;

import java.net.DatagramPacket;

/**
 * <p>Title: ProxyManager</p>
 * <p>Description: This class uses inits major components. Uses the
 * UDPComManager to listen for incoming udp messages and creates protocol
 * handlers to deal with them.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Printronix</p>
 * @author Christopher M. King (kingc@uci.edu)
 * @version 0.5
 */

public class ProxyManager
    implements ProxyConstants
{
  private UDPComManager udpManager;
  private POMESComManager xmlManager;
  private ProxySecurityManager security;
  private ProxyLog proxyLog;
  private ProxySessionManager sessionTable;
  private ProxyConfig config;

  public ProxyManager(ProxyConfig conf)
  {
     System.out.println(START_MESSAGE);

     config  = conf;

     proxyLog   = new ProxyLog(config.getLogFileName());
     udpManager = new UDPComManager(this, config.getListenPort());
     xmlManager = new POMESComManager(this, config.getPomesIP(),
                                                 config.getPomesPort(),
                                                 config.getName(),
                                                 config.getPassword());
     sessionTable = new ProxySessionManager(xmlManager);
     security     = new ProxySecurityManager();

     security.setCommunityName(config.getCommunity());
  }

  /**
   * Begins listening for incoming udp messages
   */
  public void start()
  {
    udpManager.listen(config.getListenPort());
  }

  /**
   * Helper fucntion that handles incoming UDPPackets, normally called internally
   * when the a UDPPacket is recieved by the ProxyManager that creates a new
   * protocol thread to deal with it.
   *
   * @param udpPacket A UDPPacket to be dealt with
   */
  public void recievedUDP(DatagramPacket udpPacket)
  {
    if(config.getDebug())
    {
      String temp = "";

      temp += "RECIEVED UDP MESSAGE:";
      temp += "   " + "ip = " + udpPacket.getAddress();
      temp += "   " + "port = " + udpPacket.getPort();

      System.out.println(temp);
      proxyLog.logEvent(temp);
    }
    ProtocolHandler p = new ProtocolHandler(udpPacket,udpManager,security,sessionTable,proxyLog,config.getResponseTimeout(), config.getTranslationFile());
    p.start();
  }
  public void recievedXML(String xml)
  {
    if(config.getDebug())
    {
      String temp = "";

      temp += "RECIEVED POMES RESPONSE:\n";
      temp += xml;
      System.out.print(temp);
      proxyLog.logEvent(temp);
    }
  }
}
