package dataservices;

import java.util.*;
import java.lang.*;
import java.net.InetAddress;

import communicationservices.*;
import dataservices.*;

/**
 * <p>Title: </p>
 * <p>Description: A table of Sessions </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Christopher M. King (kingc@uci.edu)
 * @version 0.5
 */

public class ProxySessionManager extends Thread
{
  Vector openSessions = new Vector();
  private boolean debug = false;
  POMESComManager pomesComManager;

  public ProxySessionManager(POMESComManager pCM)
  {
    pomesComManager = pCM;
    this.start();
  }

  /**
   * Starts a thread for the ProxySessionManager that counts down time by one
   * second at a time, calling the tick() function on every Session it is
   * managing and timing out Sessions that have expired (i.e. have had no
   * activity for over a specified time)
   *
   * @see Session
   */
  public void run()
  {
    while(true)
    {
      tick();
      try
      {
        this.sleep(1000);
      }
      catch(Exception e)
      {
        System.out.println();
      }
    }
  }

  public ProxySession openProxySession(InetAddress ip)
  {
    POMESClient pc = pomesComManager.createPOMESClient();
    ProxySession s = new ProxySession(pc, ip);
    openSessions.add(s);
    return s;
  }

  public void closeProxySession(InetAddress ip, int port)
  {
    ProxySession curSession;
    for(int i=0; i<openSessions.size(); i++)
    { curSession = (ProxySession)openSessions.elementAt(i);
      if(curSession.getIP() == ip)
        openSessions.remove(curSession);
    }
  }
  public ProxySession findProxySession(InetAddress ip)
  {
    ProxySession curSession = null;
    for(int i=0; i<openSessions.size(); i++)
    { curSession = (ProxySession)openSessions.elementAt(i);
      if(curSession.getIP().equals(ip))
        return curSession;
    }
    return null;
  }

  private void tick()
  {
    ProxySession curSession;
    for(int i=0; i<openSessions.size(); i++)
    { curSession = (ProxySession)openSessions.elementAt(i);
      curSession.tick();
      if(curSession.hasExpired())
        openSessions.remove(curSession);
    }
  }

  public String toString()
  {
    String temp = "";
    for(int i=0; i<openSessions.size(); i++)
      temp += openSessions.elementAt(i).toString();
    return temp;
  }
}