package dataservices;

import snmp.*;
import java.net.InetAddress;
import java.io.*;
import java.util.*;

// wrapper class for SNMP package
// @author Allen Liao
public class SNMPMessageInfo
{
    public static final int GET = 1;
    public static final int GETNEXT = 2;
    public static final int SET = 3;
    public static final int ERROR = -1;

    public Vector OIDs;

    private int port = -1;
    private InetAddress address = null;
    private SNMPMessage message;
    SNMPPDU pdu;

    public SNMPMessageInfo(InetAddress add, int p, SNMPMessage in)
    {
        message = in;
        port = p;
        address = add;

        try
        {
            pdu = message.getPDU();
        }
        catch(Exception e){}

        OIDs = this.extractOIDs();
    }

    public SNMPMessage getMessage()
    {
        return message;
    }

    public int getRequestID()
    {
      try
      {
        return message.getPDU().getRequestID();
      }
      catch(SNMPBadValueException e)
      {
        System.out.println(e.toString());
      }
      return 0;
    }

    public int getPort()
    {
        return port;
    }

    public InetAddress getIP()
    {
        return address;
    }

    public SNMPInteger getVersion()
    {
        return message.getVersion();
    }

    public Vector getOIDs()
    {
      return OIDs;
    }

    public Vector extractOIDs()
    {

      Vector oids = new Vector();
      int buf = 0;
      SNMPSequence varList = null;
      try
      { varList = message.getPDU().getVarBindList();
      }
      catch(SNMPBadValueException e){}

      for (int i=0; i<varList.size(); i++)
      {
        //parse oid, supply the value for that oid
        SNMPObjectIdentifier oid = (SNMPObjectIdentifier)(((SNMPSequence)varList.getSNMPObjectAt(i)).getSNMPObjectAt(0));
        oids.add(oid.toString());
      }

      return oids;
    }

    public SNMPOctetString getCommunity()
    {
        return message.getCommunity();
    }

    public String toString()
    {
      try
      {
        return this.getMessage().getPDU().getVarBindList().getSNMPObjectAt(0).toString();
      }
      catch(Exception e)
      {
        System.out.println(e);
      }
      return "ERROR";
    }

    public int getType()
    {
        byte pduType = pdu.getPDUType();
        return (pduType == SNMPBERCodec.SNMPGETREQUEST ? GET :
                pduType == SNMPBERCodec.SNMPGETNEXTREQUEST ? GETNEXT :
                pduType == SNMPBERCodec.SNMPSETREQUEST ? SET : ERROR);
    }
}