package dataservices;

import java.io.*;
import java.net.*;
import java.util.*;

// takes in attributes names and values to create POMES commands
public class XMLCommandProcessor
{
  private static final String CMD_HEADER =
                            "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n"
                          + "<POM-API>\n";

  private static final String CMD_FOOTER =
                            "</POM-API>" + (char)(06);

  public String formGetCommand(Vector attrNames, Vector attrVals)
  {
    this.appendCommandPrefix("Get", attrVals);
    return formCommand(attrNames, attrVals);
  }

  public String formSetCommand(Vector attrNames, Vector attrVals)
  {
    this.appendCommandPrefix("Set", attrVals);
    return formCommand(attrNames, attrVals);
  }

  public String formDeleteCommand(Vector attrNames, Vector attrVals)
  {
    this.appendCommandPrefix("Delete", attrVals);
    return formCommand(attrNames, attrVals);
  }

   public String formTableSetCommand(Vector cmdAttrNames,
                                     Vector cmdAttrVals,
                                     String entName,
                                     Vector entAttrNames,
                                     Vector entAttrValues)
  {
    this.appendCommandPrefix("Set", cmdAttrVals);

    String xml = "";
    xml += CMD_HEADER;
    xml += "  <COMMAND " + this.formAttrPairs(cmdAttrNames, cmdAttrVals) + ">\n";
    xml += this.formTableEntry(entName, entAttrNames, entAttrValues);
    xml += "  </COMMAND>\n";
    xml += CMD_FOOTER;

    return xml;
  }

  // i.e. <COMMAND name="Authenticate" user="jmorrison" password ="bambi"/>
  public String formCommand(Vector attrNames, Vector attrVals)
  {
    String xml = "";
    xml += CMD_HEADER;
    xml += "  <COMMAND " + this.formAttrPairs(attrNames, attrVals) + "/>\n";
    xml += CMD_FOOTER;

    return xml;
  }

  // i.e. <SMTPAlertEntry  entryid="1" emaddress="someone@email.com" mask="101010"  />
  public String formTableEntry(String entName, Vector antAttrNames, Vector entAttrVals)
  {
    String xml = "";
    xml += "    <" + entName + " " + this.formAttrPairs(antAttrNames, entAttrVals) + " />\n";
    return xml;
  }

  // i.e. "SMTPServer" converted to "SetSMTPServer"
  private void appendCommandPrefix(String cmdPrefix, Vector attrVals)
  { // give command a "Get" prefix
    String cmd = (String)attrVals.elementAt(0); // First attrVal is the
    cmd = cmdPrefix + cmd;                      // command name
    attrVals.setElementAt(cmd,0);
  }

  // i.e. name="Authenticate" user="jmorrison" password ="bambi"
  private String formAttrPairs(Vector attrNames, Vector attrVals)
  {
    String attrs = "";
    for(int i=0; i<attrNames.size() && i<attrVals.size(); i++)
    {
      attrs += (String)attrNames.elementAt(i);
      attrs += "=";
      attrs += "\"" + (String)attrVals.elementAt(i) + "\" ";
    }
    return attrs;
   }
}