package dataservices;
/**
 * <p>Title: XMLResponseParser</p>
 * <p>Description: Used by the SNMPMessageHandler</p>
 *
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Raymundo Anaya (anayar@uci.edu)
 * @version 1.0
 */

import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;

class XMLResponseParser
{
    String nameAttribute = "";
    String successAttribute = "";
    String valueAttribute = "";
    String tokenAttribute = "";
    String responseObjectTag = "";
    boolean responseTag; //flag to know if response tag has been seen
    Vector tableVals;
    Vector tableNames;
    int counter;

    public XMLResponseParser()
    {
        tableVals = new Vector();
        tableNames = new Vector();
        counter = 0;
        responseTag = false;
    }

    public String getValue()
    {   return valueAttribute;  }

    public String getName()
    {   return nameAttribute;   }

    public String getToken()
    {   return tokenAttribute;  }

    public boolean getSuccess()
    {   return successAttribute.equals("true"); }

    public Vector getTableAttrVals()
    {   return tableVals;  }

    public Vector getTableAttrNames()
    {   return tableNames;}

    public String getResponseObjectTag()
    { return responseObjectTag;}

    public void XMLparse( String s )
    {
        String newToken;
        StringTokenizer tokenizer = new StringTokenizer( s, ">" );

        int numTokens = tokenizer.countTokens();

        for( int i = 1; i < numTokens; ++i )
        {
            newToken = tokenizer.nextToken().trim();

            if ( responseTagCheck( newToken, "> ") )
            {
                if(responseTag)//reached end tag
                {
                    i = numTokens; //end loop
                    responseTag = false;
                }
                else
                {
                    tokenParse( newToken, "= \"");
                    responseTag = true;
                }
            }
            else if( responseTag )//response tag has been parsed
                containParse( newToken );
        }
    }

    private void containParse( String s )
    {
        if( s.charAt(0) == '<' ) //data present
            tableParse( s, "=\" ");

        else
        {
            String newToken;
            StringTokenizer tokenizer = new StringTokenizer( s, "<" );
            valueAttribute = tokenizer.nextToken().trim();
        }
    }

    private void tableParse( String s, String delim)
    {
        Vector entryVals = new Vector();
        Vector entryNames = new Vector();

        String newToken;
        StringTokenizer tokenizer = new StringTokenizer( s, delim );
        int numTokens = tokenizer.countTokens();
        responseObjectTag = tokenizer.nextToken().trim();
        // Quick Fix: chops of the leading '<'
        responseObjectTag = responseObjectTag.substring(1, responseObjectTag.length());
        for( int i = 2; i < numTokens; ++i )
        {
          entryNames.add( tokenizer.nextToken().trim());//skip element name
          entryVals.add(tokenizer.nextToken().trim());//enter actual value
          ++i;
        }
        tableVals.add(counter, entryVals);
        tableNames.add(counter, entryNames);
        counter++;
    }

    private void tokenParse( String s, String delim )
    {
        String newToken;
        StringTokenizer tokenizer = new StringTokenizer( s, delim );

        int numTokens = tokenizer.countTokens();

        for( int i = 1; i < numTokens; ++i )
        {
            newToken = tokenizer.nextToken().trim();

            if( newToken.equals("name"))
            {
                nameAttribute = tokenizer.nextToken().trim();
                ++i;
            }
            else if(newToken.equals("token"))
            {
                tokenAttribute = tokenizer.nextToken().trim();
                ++i;
            }
            else if(newToken.equals("success"))
            {
                successAttribute = tokenizer.nextToken().trim();
                ++i;
            }
        }
    }

    private boolean responseTagCheck( String s, String delim )
    {
        StringTokenizer tokenizer = new StringTokenizer( s, delim );
        String newToken = tokenizer.nextToken().trim();

        if( newToken.equals("<RESPONSE") || newToken.equals("</RESPONSE") )
            return true;
        return false;
    }
}